@@ -9,19 +9,20 @@ use warnings;
 no  warnings qw( once redefine uninitialized );
 
 use Text::Balanced; 
-use Data::Dump::Streamer;
-use Pugs::Runtime::Rule;
+use Data::Dumper;
+use Pugs::Runtime::LrepRule;
+use Pugs::Runtime::LrepMatch;
 #use Pugs::Runtime::Grammar; -- MOP 
 
 use vars qw( @rule_terms );
-use base 'Pugs::Grammar::Base';
+use base 'Pugs::Grammar::LrepBase';
 use Pugs::Grammar::Rule::Rule;   # compiled with lrep
 
 sub code {
     my $grammar = shift;
     return $grammar->no_match unless $_[0];
     my ($extracted,$remainder) = Text::Balanced::extract_codeblock( $_[0] );
-    return Pugs::Runtime::Match->new( { 
+    return Pugs::Runtime::LrepMatch->new( { 
         bool  => ( $extracted ne '' ),
         match => $extracted,
         tail  => $remainder,
@@ -33,7 +34,7 @@ sub literal {
     return $grammar->no_match unless $_[0];
     my ($extracted,$remainder) = Text::Balanced::extract_delimited( $_[0], "'" );
     $extracted = substr( $extracted, 1, -1 ) if length($extracted) > 1;
-    return Pugs::Runtime::Match->new( { 
+    return Pugs::Runtime::LrepMatch->new( { 
         bool  => ( $extracted ne '' ),
         match => $extracted,
         tail  => $remainder,
@@ -45,7 +46,7 @@ sub metasyntax {
     return $grammar->no_match unless $_[0];
     my ($extracted,$remainder) = Text::Balanced::extract_bracketed( $_[0], "<..>" );
     $extracted = substr( $extracted, 1, -1 ) if length($extracted) > 1;
-    return Pugs::Runtime::Match->new( { 
+    return Pugs::Runtime::LrepMatch->new( { 
         bool  => ( $extracted ne '' ),
         match => $extracted,
         tail  => $remainder,
@@ -64,7 +65,7 @@ BEGIN {
         sub{ 
             # warn "Trying $method\n";
             my $match = Pugs::Grammar::Rule->$method(@_);
-            #warn "Match $method ".Dump($match) if $match->{bool};
+            #warn "Match $method ".Dumper($match) if $match->{bool};
             return $match;
         }
     }
